"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from cfnlint.rules import CloudFormationLintRule
from cfnlint.rules import RuleMatch


class InterfaceParameterExists(CloudFormationLintRule):
    """Check if Metadata Interface parameters exist"""
    id = 'W4001'
    shortdesc = 'Metadata Interface parameters exist'
    description = 'Metadata Interface parameters actually exist'
    source_url = 'https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudformation-interface.html'
    tags = ['metadata']

    valid_keys = [
        'ParameterGroups',
        'ParameterLabels'
    ]

    def match(self, cfn):
        """Check CloudFormation Metadata Parameters Exist"""

        matches = []

        strinterface = 'AWS::CloudFormation::Interface'
        parameters = cfn.get_parameter_names()
        metadata_obj = cfn.template.get('Metadata', {})
        if metadata_obj:
            interfaces = metadata_obj.get(strinterface, {})
            if isinstance(interfaces, dict):
                # Check Parameter Group Parameters
                paramgroups = interfaces.get('ParameterGroups', [])
                if isinstance(paramgroups, list):
                    for index, value in enumerate(paramgroups):
                        if 'Parameters' in value:
                            for paramindex, paramvalue in enumerate(value['Parameters']):
                                if paramvalue not in parameters:
                                    message = 'Metadata Interface parameter doesn\'t exist {0}'
                                    matches.append(RuleMatch(
                                        ['Metadata', strinterface, 'ParameterGroups',
                                         index, 'Parameters', paramindex],
                                        message.format(paramvalue)
                                    ))
                paramlabels = interfaces.get('ParameterLabels', {})
                if isinstance(paramlabels, dict):
                    for param in paramlabels:
                        if param not in parameters:
                            message = 'Metadata Interface parameter doesn\'t exist {0}'
                            matches.append(RuleMatch(
                                ['Metadata', strinterface, 'ParameterLabels', param],
                                message.format(param)
                            ))

        return matches



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
import json
from cfnlint.rules import CloudFormationLintRule
from cfnlint.rules import RuleMatch


class InstanceEngine(CloudFormationLintRule):
    """Check if Resources RDS Instance Size is compatible with the RDS type"""
    id = 'E3040'
    shortdesc = 'RDS DB Instance Engine is valid'
    description = 'Check the RDS DB Instance Engine is valid'
    source_url = 'https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/API_CreateDBInstance.html'
    tags = ['resources', 'rds']

    valid_engines = [
        'aurora',
        'aurora-mysql',
        'aurora-postgresql',
        'mariadb',
        'mysql',
        'oracle-ee',
        'oracle-se2',
        'oracle-se1',
        'oracle-se',
        'postgres',
        'sqlserver-ee',
        'sqlserver-se',
        'sqlserver-ex',
        'sqlserver-web'
    ]

    def __init__(self):
        """Init"""
        super(InstanceEngine, self).__init__()
        self.resource_property_types = ['AWS::RDS::DBInstance']

    def check_value(self, value, path):
        """Check engine"""
        if value.lower() not in self.valid_engines:
            message = 'RDS Engine "{0}" must be one of {1}'
            return [RuleMatch(path, message.format(value, json.dumps(self.valid_engines)))]

        return []

    def match_resource_properties(self, properties, _, path, cfn):
        """Match for sub properties"""
        matches = []
        matches.extend(
            cfn.check_value(
                obj=properties, key='Engine',
                path=path[:],
                check_value=self.check_value
            ))
        return matches



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from cfnlint.rules import CloudFormationLintRule
from cfnlint.rules import RuleMatch


class UpdateReplacePolicyDeletionPolicyOnStatefulResourceTypes(CloudFormationLintRule):
    """Check for UpdateReplacePolicy / DeletionPolicy"""
    id = 'I3011'
    shortdesc = 'Check stateful resources have a set UpdateReplacePolicy/DeletionPolicy'
    description = 'The default action when replacing/removing a resource is to ' \
                  'delete it. This check requires you to explicitly set policies'
    source_url = 'https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-attribute-deletionpolicy.html'
    tags = ['resources', 'updatereplacepolicy', 'deletionpolicy']

    def match(self, cfn):
        """Check for UpdateReplacePolicy / DeletionPolicy"""
        matches = []

        likely_stateful_resource_types = ['AWS::CloudFormation::Stack',
                                          'AWS::Backup::BackupVault',
                                          'AWS::Cognito::UserPool',
                                          'AWS::DocDB::DBCluster',
                                          'AWS::DocDB::DBInstance',
                                          'AWS::DynamoDB::Table',
                                          'AWS::EC2::Volume',
                                          'AWS::EFS::FileSystem',
                                          'AWS::EMR::Cluster',
                                          'AWS::ElastiCache::CacheCluster',
                                          'AWS::ElastiCache::ReplicationGroup',
                                          'AWS::Elasticsearch::Domain',
                                          'AWS::FSx::FileSystem',
                                          'AWS::Logs::LogGroup',
                                          'AWS::Neptune::DBCluster',
                                          'AWS::Neptune::DBInstance',
                                          'AWS::QLDB::Ledger',
                                          'AWS::RDS::DBCluster',
                                          'AWS::RDS::DBInstance',
                                          'AWS::Redshift::Cluster',
                                          'AWS::SDB::Domain',
                                          # 'AWS::S3::Bucket', (CloudFormation won't delete anyways)
                                         ]

        resources = cfn.get_resources()
        for r_name, r_values in resources.items():
            if r_values.get('Type') in likely_stateful_resource_types:
                if not r_values.get('DeletionPolicy') or not r_values.get('UpdateReplacePolicy'):
                    path = ['Resources', r_name]
                    message = 'The default action when replacing/removing a resource is to delete it. Set explicit values for UpdateReplacePolicy / DeletionPolicy on potentially stateful resource: %s' \
                    % '/'.join(path)
                    matches.append(RuleMatch(path, message))

        return matches
